@@ -1,5 +1,5 @@
 package MaxMind::DB::Writer::Tree::Processor::VisualizeTree;
-$MaxMind::DB::Writer::Tree::Processor::VisualizeTree::VERSION = '0.060000';
+$MaxMind::DB::Writer::Tree::Processor::VisualizeTree::VERSION = '0.080000';
 use strict;
 use warnings;
 
@@ -36,21 +36,21 @@ has _labels => (
 );
 
 sub process_node_record {
-    my $self               = shift;
-    my $node_num           = shift;
-    my $dir                = shift;
-    my $node_ip_num        = shift;
-    my $node_mask_length   = shift;
-    my $record_ip_num      = shift;
-    my $record_mask_length = shift;
-    my $record_node_num    = shift;
+    my $self                 = shift;
+    my $node_num             = shift;
+    my $dir                  = shift;
+    my $node_ip_num          = shift;
+    my $node_prefix_length   = shift;
+    my $record_ip_num        = shift;
+    my $record_prefix_length = shift;
+    my $record_node_num      = shift;
 
     $self->graph()->add_edge(
         from => $self->_label_for_node(
-            $node_num, $node_ip_num, $node_mask_length
+            $node_num, $node_ip_num, $node_prefix_length
         ),
         to => $self->_label_for_node(
-            $record_node_num, $record_ip_num, $record_mask_length
+            $record_node_num, $record_ip_num, $record_prefix_length
         ),
         label => ( $dir ? 'RIGHT' : 'LEFT' ),
     );
@@ -63,20 +63,20 @@ sub process_empty_record {
 }
 
 sub process_data_record {
-    my $self               = shift;
-    my $node_num           = shift;
-    my $dir                = shift;
-    my $node_ip_num        = shift;
-    my $node_mask_length   = shift;
-    my $record_ip_num      = shift;
-    my $record_mask_length = shift;
-    my $value              = shift;
+    my $self                 = shift;
+    my $node_num             = shift;
+    my $dir                  = shift;
+    my $node_ip_num          = shift;
+    my $node_prefix_length   = shift;
+    my $record_ip_num        = shift;
+    my $record_prefix_length = shift;
+    my $value                = shift;
 
     $self->graph()->add_edge(
         from => $self->_label_for_node(
-            $node_num, $node_ip_num, $node_mask_length
+            $node_num, $node_ip_num, $node_prefix_length
         ),
-        to => $self->_network( $record_ip_num, $record_mask_length ) . ' = '
+        to => $self->_network( $record_ip_num, $record_prefix_length ) . ' = '
             . $self->_data_record_representation($value),
         label => ( $dir ? 'RIGHT' : 'LEFT' ),
     );
@@ -85,12 +85,12 @@ sub process_data_record {
 }
 
 sub _label_for_node {
-    my $self        = shift;
-    my $node_num    = shift;
-    my $ip_num      = shift;
-    my $mask_length = shift;
+    my $self          = shift;
+    my $node_num      = shift;
+    my $ip_num        = shift;
+    my $prefix_length = shift;
 
-    my $network = $self->_network( $ip_num, $mask_length );
+    my $network = $self->_network( $ip_num, $prefix_length );
 
     return $self->_labels()->{$node_num} //=
           "Node $node_num - "
@@ -112,14 +112,14 @@ sub _data_record_representation {
 }
 
 sub _network {
-    my $self        = shift;
-    my $ip_num      = shift;
-    my $mask_length = shift;
+    my $self          = shift;
+    my $ip_num        = shift;
+    my $prefix_length = shift;
 
     return Net::Works::Network->new_from_integer(
-        integer     => $ip_num,
-        mask_length => $mask_length,
-        version     => $self->ip_version(),
+        integer       => $ip_num,
+        prefix_length => $prefix_length,
+        version       => $self->ip_version(),
     );
 }
 
